@@ -5,7 +5,7 @@
 
 package XML::Compile::Schema::XmlWriter;
 use vars '$VERSION';
-$VERSION = '0.13';
+$VERSION = '0.14';
 
 use strict;
 use warnings;
@@ -185,6 +185,8 @@ sub create_complex_element
           {   next unless $tag =~ m/^\{([^}]*)\}(.*)$/;
               my ($ns, $type) = ($1, $2);
               my $value = delete $data->{$tag};
+              defined $value or next;
+
               my $any
                = !ref $value                         ? undef
                : $value->isa('XML::LibXML::Attr')    ? $any_attr
@@ -276,7 +278,7 @@ sub create_simple_element
 }
 
 sub builtin_checked
-{   my ($path, $args, $type, $def) = @_;
+{   my ($path, $args, $node, $type, $def) = @_;
     my $check  = $def->{check};
     defined $check
        or return builtin_unchecked(@_); 
@@ -286,21 +288,22 @@ sub builtin_checked
 
       defined $format
     ? sub { defined $_[1] or return undef;
-            my $value = $format->($_[1]);
+            my $value = $format->($_[1], $node);
             return $value if defined $value && $check->($value);
             $value = $err->($path, $_[1], "illegal value for $type");
-            defined $value ? $format->($value) : undef;
+            defined $value ? $format->($value, $node) : undef;
           }
     : sub { return $_[1] if !defined $_[1] || $check->($_[1]);
             my $value = $err->($path, $_[1], "illegal value for $type");
-            defined $value ? $format->($value) : undef;
+            defined $value ? $format->($value, $node) : undef;
           };
 }
 
 sub builtin_unchecked
-{   my $format = $_[3]->{format};
+{   my $format = $_[4]->{format};
+    my $node   = $_[2];
       defined $format
-    ? sub { defined $_[1] ? $format->($_[1]) : undef }
+    ? sub { defined $_[1] ? $format->($_[1], $node) : undef }
     : sub { $_[1] }
 }
 
@@ -423,7 +426,7 @@ sub attribute_fixed_optional
         };
 }
 
-# anyAttribute
+# any
 
 sub anyAttribute
 {   my ($path, $args, $handler, $yes, $no, $process) = @_;
@@ -441,6 +444,26 @@ sub anyAttribute
         };
 }
 
+sub anyElement
+{   my ($path, $args, $handler, $yes,$no, $process, $min,$max) = @_;
+    my %yes = map { ($_ => 1) } @{$yes || []};
+    my %no  = map { ($_ => 1) } @{$no  || []};
+    my $err = $args->{err};
+
+    defined $handler or return sub { () };
+
+    sub { my ($doc, $key, $ns, $type, $values) = @_;
+          my @values = ref $values eq 'ARRAY' ? @$values : $values;
+          foreach my $value (@values)
+          {  my $vns = $value->namespaceURI;
+             $vns eq $ns or $err->($path, $vns, "value name-space must be $ns");
+             return $yes{$ns} ? $value : undef  if keys %yes;
+             return $no{$ns}  ? undef  : $value if keys %no;
+          }
+          @values;
+        };
+}
+
 
 1;
 
